#!/usr/local/bin/perl

use strict;
use DBI;
use Parallel::ForkManager;

my $DEBUG = 0;

my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open config: $!\n";
while(<CONFIG>){
   next if(/^[#;]/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;


my $pm = new Parallel::ForkManager(10);

$pm->run_on_wait(
	sub {
		print "**** Maximum number of proccesses spawned, sleeping for 2 seconds ...\n" if($DEBUG);
	},
	2
);

$pm->run_on_start(
	sub { my ($pid,$ident)=@_;
		print "** $ident started, pid: $pid\n" if($DEBUG);
	}
);

$pm->run_on_finish(
	sub { my ($pid, $exit_code, $ident) = @_;
		print "** $ident just got out of the pool ".
		      "with PID $pid and exit code: $exit_code\n" if($DEBUG);
	}
);

my $dbh;
until($dbh){
	$dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
	warn "$DBI::errstr\n" if($DBI::errstr);
	sleep 1;
}

my $sql = 'SELECT netid FROM data WHERE status = 0 AND';
$sql .= ' enable_monitoring = 1 ORDER BY netid DESC';
my $sth = $dbh->prepare($sql);
$sth->execute;
my @rows;
while(my $row = $sth->fetchrow_arrayref){
	push @rows, $row->[0];
}
$sth->finish;
$dbh->disconnect;

while(@rows){
	my $id1 = shift @rows;
	my $id2 = shift @rows;
	my $id3 = shift @rows;
	my $id4 = shift @rows;
	my $id5 = shift @rows;
	my $id6 = shift @rows;

	print "[PARENT $$] Assigning task: [$id1], [$id2], [$id3], [$id4], [$id5], [$id6]\n" if($DEBUG);
	my $pid = $pm->start("$id1,$id2,$id3,$id4,$id5,$id6") and next;
	#This code is the child process
	system("$cfg{home_dir}/data/scripts/networkchk.pl", $id1, $id2, $id3, $id4, $id5, $id6);
	$pm->finish;
}

$pm->wait_all_children;
