#!/usr/local/bin/perl

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use HTML::Template;
use CGI qw(:standard);
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use IPAnalyze::Auth;
use IPAnalyze::Misc;
use IPAnalyze::IPmath;
use IPAnalyze::Taint;


my %cfg;
my @cfg_keys; # for displaying them in order
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open config: $!\n";
while(<CONFIG>){
	s/^(.+)\s*#.*/$1/;
	next if(/^$/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
		push @cfg_keys, $1;
      $cfg{$1} = $2;
   }
}
close CONFIG;

my $version = $cfg{'version'};
my $html_dir = $cfg{'html_dir'};
my $script_root = "$ENV{'DOCUMENT_ROOT'}/cgi-bin";
my $script = $ENV{'SCRIPT_NAME'};
my $ipa = 'http://' . $cfg{http_host} . '/cgi-bin/ipanalyze.mpl';

my $dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
my $q = CGI->new();
my $auth = IPAnalyze::Auth->new();
my $misc = IPAnalyze::Misc->new();
my $ipmath = IPAnalyze::IPmath->new();
my $taint = IPAnalyze::Taint->new();

my ($username,$access,@error) = $auth->check_access($dbh,$q,$cfg{app});
if (@error){
	print $q->header(-expires=>'now');
	my $template = HTML::Template->new(filename => "$html_dir/login.html");
   $template->param(SCRIPT => $script);
   $template->param(VERSION => $version);
	$template->param(REDIRECT => $script);
   print $template->output;
	foreach my $error (@error){
		print "<p align=\"center\"><i><font color=\"#FF0000\"><b>$error</b></font></i></p>";
	}
	exit;
}

unless($access >= 4){
	my @error = ("Access denied.");
	print $q->header(-expires=>'now');
	$misc->show_error($html_dir,\@error);
	exit;
}

if(my $mode=$q->param('mode')){
	print $q->header(-expires=>'now');
	my $template = HTML::Template->new(filename => "$html_dir/main.html");
	$template->param(SCRIPT => $script);
	$template->param(IPMANAGE => 1);
	$template->param(IPMONITOR => 1);
	$template->param(VERSION => $version);
	$template->param(USERNAME => $username);
	$template->param(OUR_COMPANY => $cfg{our_name});

	if($mode eq 'cache'){
		unless($access >= 5){
			my @error;	
			push @error, "Access denied.";
			$misc->show_error($html_dir,\@error);
			exit;
		}
		$template->param(MANAGE_CACHE => 1);
		if(my $function=$q->param('function')){
			if($function eq 'Dump IP Cache'){
				$dbh->do("TRUNCATE TABLE skip_cache");

				$template->param(IP_DONE => 1);
			}elsif($function eq 'Dump OC8 Cache'){
				foreach my $table (qw/oc8_cache_ids oc8_cache_data/){
					$dbh->do("TRUNCATE TABLE ${table}");
				}
				$template->param(OC8_DUMP => 1);
			}elsif($function eq 'Refresh OC8 Cache'){
				system("$cfg{home_dir}/data/scripts/houston/refresh_cache.pl","-f");
				$template->param(OC8_REFRESH => 1);
			}
		}

	}elsif($mode eq 'config'){
		unless($access >= 5){
			my @error;	
			push @error, "Access denied.";
			$misc->show_error($html_dir,\@error);
			exit;
		}
		$template->param(MANAGE_CONFIG => 1);
		if($q->param('submit')){
			my @error;
			foreach my $key (@cfg_keys){
				unless($q->param($key)){
					push @error, "Confuzzled: no $key received";
				}
			}
			unless(-d $q->param('home_dir')){
				push @error, "home_dir doesnt exist";
			}
			my $h = $q->param('html_dir');
			if(-d $h){
				unless(-s "$h/main.html"){
					push @error, "$h/main.html doesnt exist";
				}
			}else{
				push @error, "html_dir doesnt exist";
			}
			if(@error){
				$misc->show_error($html_dir,\@error);
				exit;
			}


			open(F, ">/export/web/ipanalyze.example.com/data/ipanalyze.conf") || die "cannot write to config: $!\n";
			foreach my $key (@cfg_keys){
				if((($key eq 'd_dbpw') || ($key eq 'c_dbpw')) && ($q->param($key) eq '********')){
					print F "$key => ", $cfg{$key}, "\n";
				}else{
					print F "$key => ", $q->param($key), "\n";
					$cfg{$key} = $q->param($key);
				}
			}
			close F;


		}
		my @data;
		foreach my $key (@cfg_keys){
			my $type = ($key =~ /dbpw$/) ? 'password' : 'text';
			my $value = ($key eq 'd_dbpw' || $key eq 'c_dbpw') ? '********' : $cfg{$key};
				
			push @data, {KEY => $key, VALUE => $value};
		}
		$template->param(CONFIG => \@data);

	}elsif($mode eq 'networks'){

		my %areas = $misc->get_areas($dbh);

		my (@error,$sql,$must_taint,$notaint);
		
		if($q->param('p_update') &&
         (my $id=$q->param('id')) && (my $netblock=$q->param('netblock')) &&
		   (my $cidr=$q->param('cidr')) && defined(my $area=$q->param('area')) &&
		   defined(my $allowed=$q->param('allowed'))){

			unless($id == int($id)){
				push @error, "Invalid ID: $id\n";
			}
			unless($cidr =~ /^(?:19|2[012345])$/){
				push @error, "Invalid CIDR: $cidr\n";
			}
			unless($area =~ /^\d{1,5}$/){
				push @error, "Invalid Area: $area\n";
			}
			if(@error){
				$misc->show_error($html_dir,\@error);
				exit;
			}

			my $sqld = 'SELECT netblock,cidr FROM parent_networks WHERE id = ' . $dbh->quote($id);
			my $sth = $dbh->prepare($sqld);
			$sth->execute;
			my $rowd = $sth->fetchrow_arrayref;
			if($rowd->[0] eq $netblock && $rowd->[1] eq $cidr){
				$notaint=1;
			}

			my $sqlc = 'SELECT id FROM parent_networks WHERE netblock = ' . $dbh->quote($netblock);
			$sth = $dbh->prepare($sqlc);
			$sth->execute;
			my $rowc = $sth->fetchrow_arrayref;
			if((! $rowc->[0]) || ($rowc->[0] == $id)){
				my $sqlb = 'SELECT parent_id FROM parent_networks WHERE id = ' . $dbh->quote($id);
				my $sth = $dbh->prepare($sqlb);
				$sth->execute;
				my $rowb = $sth->fetchrow_arrayref;
				my $pid = $rowb->[0];
	
				# make sure the parent is still correct
				# make sure netblock/cidr is part of parent_id, also proving $netblock/$cidr is valid..
				my $sqla = 'SELECT netblock,cidr FROM grandparent_networks WHERE id = ' . $dbh->quote($pid);
				$sth = $dbh->prepare($sqla);
				$sth->execute;
				my $row = $sth->fetchrow_arrayref;
				if($row->[0]){
					# make sure the netblock/cidr is part of the range we just selected
					my @n = $ipmath->get_netblocks($row->[0],$row->[1],$cidr);
					unless(grep /^$netblock$/, @n){
						push @error, "${netblock}/${cidr} doesnt appear to be part of $row->[0]/$row->[1], or is invalid";
					}
				}else{
					push @error, "Invalid parent_id for new netblock: $pid";
				}
			}else{
				push @error, "Parent $netblock already exists: $rowc->[0] vs $id\n";
			}

			if(@error){
				$misc->show_error($html_dir,\@error);
				exit;
			}else{
				$sql = 'UPDATE parent_networks SET netblock = ' . $dbh->quote($netblock);
				$sql .= ', cidr = ' . $dbh->quote($cidr) . ', area = ' . $dbh->quote($area);
				$sql .= ', allow_use = ';
				if($area == 0){
					$sql .= '0'; # makes sense
				}else{
					$sql .= $dbh->quote($allowed);
				}
				$sql .= ' WHERE id = ' . $dbh->quote($id);

				my $sqlb = 'DELETE FROM skip_cache WHERE parent_id = ' . $dbh->quote($id);
				my $sthb = $dbh->prepare($sqlb);
				$sthb->execute;

				unless($notaint){
					$must_taint=1; # really only if netblock/cidr is changed - not area
				}
			}

		}elsif(($q->param('gp_delete') || $q->param('p_delete')) &&
		       (my $id=$q->param('id'))){

			unless($id == int($id)){
				push @error, "Invalid ID: $id";
			}

			if(@error){
				$misc->show_error($html_dir,\@error);
				exit;
			}
			
			my $sqla = 'SELECT COUNT(*) FROM ';
			$sqla .= ($q->param('gp_delete')) ? 'parent_networks' : 'data';
			$sqla .= ' WHERE parent_id = ' . $dbh->quote($id);
			my $sth = $dbh->prepare($sqla);
			$sth->execute;
			my $row = $sth->fetchrow_arrayref;
			if($row->[0]){
				my $what = ($q->param('gp_delete')) ? 'Grandparent' : 'Parent';
				push @error, "$what is in use";
				$misc->show_error($html_dir,\@error);
				exit;
			}

			$sql = 'DELETE FROM ';
			$sql .= 'grand' if($q->param('gp_delete'));
			$sql .= 'parent_networks WHERE id = ' . $dbh->quote($id);

			$must_taint=1;

		}elsif($q->param('gparent') &&
		       (my $netblock=$q->param('new_gparent')) && (my $cidr=$q->param('gp_cidr'))){
			$sql = 'INSERT INTO grandparent_networks (netblock,cidr) VALUES (';
			$sql .= $dbh->quote($netblock) . ',' . $dbh->quote($cidr) . ')';

			my $net = $ipmath->get_netblock($netblock,$cidr);
			if($net eq $netblock){
				unless($cidr >= 8 && $cidr <= 19){
					push @error, "Invalid Grandparent CIDR: $cidr";
				}
				foreach my $dec (split /\./, $netblock){
					unless($dec >= 0 && $dec <= 255){
						push @error, "Invalid netblock: $netblock\n";
					}
				}
			}else{
				push @error, "Invalid network: $netblock/$cidr\n";
			}
		}elsif($q->param('parent') &&
			    (my $netblock=$q->param('new_parent')) && (my $cidr=$q->param('p_cidr')) &&
		       (my $pid=$q->param('pid')) && defined(my $area=$q->param('area')) &&
		       defined(my $allowed=$q->param('allowed'))){
 
			my $sqla = 'SELECT COUNT(*) FROM parent_networks WHERE netblock = ' . $dbh->quote($netblock);
			my $stha = $dbh->prepare($sqla);
			$stha->execute;
			my $row = $stha->fetchrow_arrayref;
			if($row->[0]){
				push @error, "$netblock already a parent\n";
			}else{

				# make sure netblock/cidr is part of parent_id, also proving $netblock/$cidr is valid..
				$sqla = 'SELECT netblock,cidr FROM grandparent_networks WHERE id = ' . $dbh->quote($pid);
				$stha = $dbh->prepare($sqla);
				$stha->execute;
				$row = $stha->fetchrow_arrayref;
				if($row->[0]){
					# make sure the netblock/cidr is part of the range we just selected
					my @n = $ipmath->get_netblocks($row->[0],$row->[1],$cidr);
					unless(grep /^$netblock$/, @n){
						push @error, "${netblock}/${cidr} doesnt appear to be part of $row->[0]/$row->[1], or is invalid";
					}
				}else{
					push @error, "Invalid Parent_id: $pid";
				}
	
				unless($cidr >= 19 && $cidr <= 26){
					push @error, "Invalid parent CIDR: $cidr";
				}
				foreach my $dec (split /\./, $netblock){
					unless($dec >= 0 && $dec <= 255){
						push @error, "Invalid netblock: $netblock\n";
					}
				}
	
				unless(exists($areas{id}{$area})){
					push @error, "Invalid Area: $area";
				}
				unless($allowed eq '0' || $allowed eq '1'){
					push @error, "Invalid Allowed: $allowed";
				}
			}

			$sql = 'INSERT INTO parent_networks (parent_id,netblock,cidr,area,allow_use) VALUES (';
			$sql .= $dbh->quote($pid) . ',' . $dbh->quote($netblock) . ',' . $dbh->quote($cidr);
			$sql .= ',' . $dbh->quote($area) . ',';
			if($area == 0){
				$sql .= '0'; # force not allowed if unassigned area
			}else{
				$sql .= $dbh->quote($allowed);
			}
			$sql .= ')';

			warn "sql: $sql\n";
		}
		if(@error){
      	$misc->show_error($html_dir,\@error);
			exit;
		}
		if($sql){
			my $sth = $dbh->prepare($sql);
			$sth->execute;
			sleep 1; # or else our page may not display the newest results
		}

		if($must_taint){
			my @errors = $taint->taint_parents($dbh,$ipmath);
			if(@errors){
				$misc->show_error($html_dir,\@errors);
				exit;
			}
		}
	
		my (%netblocks,@areas,@gp,@gpids,@p,@notices);
		foreach my $key (sort keys %{$areas{id}}){
			my $value = $key . ' - ' . $areas{id}{$key};
			push @areas, {KEY => $value, VALUE => $key};		
		}

		$sql = 'SELECT id,netblock,cidr FROM grandparent_networks ORDER BY id';
		my $sth = $dbh->prepare($sql);
		$sth->execute;
		while(my $row = $sth->fetchrow_arrayref){
			push @gp, {ID => $row->[0], NETBLOCK => $row->[1], CIDR => $row->[2], SCRIPT => $script};
			my $key = $row->[0] . ' - ' . $row->[1] . '/' . $row->[2];
			push @gpids, {KEY => $key, VALUE => $row->[0]};
		}

		$sql = 'SELECT id,parent_id,netblock,cidr,area,allow_use FROM parent_networks';
		$sth = $dbh->prepare($sql);
		$sth->execute;
		while(my $row = $sth->fetchrow_arrayref){
			$netblocks{$row->[0]} = {parent_id => $row->[1], netblock => $row->[2],
			                         cidr => $row->[3], area => $row->[4], allow_use => $row->[5]};
		}
		foreach my $id (keys %netblocks){
			my $bgcolor = 'bgcolor="#' . $areas{color}{$netblocks{$id}{area}} . '"';
			foreach my $subid (keys %netblocks){
				if(($netblocks{$subid}{cidr} < $netblocks{$id}{cidr}) &&
				   ($netblocks{$subid}{area} eq $netblocks{$id}{area}) &&
			      ($netblocks{$subid}{allow_use} eq '1') &&
			      ($netblocks{$id}{allow_use} eq '1')){
					my @n = $ipmath->get_netblocks($netblocks{$subid}{netblock},$netblocks{$subid}{cidr},$netblocks{$id}{cidr});
					if(grep /^$netblocks{$id}{netblock}$/, @n){

						push @notices, {NOTICE => "IPAnalyze thinks ID $id can be merged into ID $subid"};
						$bgcolor = 'bgcolor="#FFFF00"';
						last;
					}
				}else{
					my $supercidr = ($netblocks{$id}{cidr} - 1);
					my $supernet = $ipmath->get_netblock($netblocks{$id}{netblock},$supercidr);
					if(($netblocks{$subid}{netblock} eq $supernet) &&
					   ($subid != $id) &&
					   ($netblocks{$subid}{cidr} eq $netblocks{$id}{cidr}) &&
				      ($netblocks{$subid}{area} eq $netblocks{$id}{area}) &&
			         ($netblocks{$subid}{allow_use} eq '1') &&
			         ($netblocks{$id}{allow_use} eq '1') &&
					   ($netblocks{$id}{cidr} < 19) ){

						push @notices, {NOTICE => "IPAnalyze thinks ID $id can be merged with ID $subid"};
	
						$bgcolor = 'bgcolor="#FFFF00"';
						last;
				
					}
				}
			}
			my $allowed = 'ALLOW_';
			$allowed .= ($netblocks{$id}{allow_use}) ? 'YES' : 'NO';
			my $select = 'CIDR_' . $netblocks{$id}{cidr};

			my @html_areas;
			foreach my $key (sort keys %{$areas{name}}){
				my $string = '<option';
				$string .= ' selected' if($areas{name}{$key} == $netblocks{$id}{area});
				$string .= ' value="' . $areas{name}{$key} . '">' . "${key}</option>";
				push @html_areas, {OPTION => $string};
			}
			push @p, {BGCOLOR => $bgcolor, ID => $id, PID => $netblocks{$id}{parent_id},
			          NETBLOCK => $netblocks{$id}{netblock}, $select => 'selected',
			          $allowed => 'selected', HTML_AREAS => \@html_areas,
			          SCRIPT => $script,
			};

		}

		my @sorted = map  { $_->[0] }
		             sort { $a->[1] <=> $b->[1] ||
		                    $a->[2] <=> $b->[2] ||
		                    $a->[3] <=> $b->[3] ||
		                    $a->[4] <=> $b->[4] }
		             map  { [ $_, split /\./, $_->{NETBLOCK} ] } @p;

		$template->param(NOTICES => \@notices) if(@notices);
		$template->param(GRANDPARENTS => \@gp);
		$template->param(PARENTS => \@sorted);
		$template->param(GPIDS => \@gpids);
		$template->param(AREAS => \@areas);
		$template->param(MANAGE_NETWORKS => 1);
		$template->param(SCRIPT => $script);
		
	}elsif($q->param('mode') eq 'users'){
		print "should use Houston users.pl\n";
	}else{
		my @error = qw/Confused/;
		$misc->show_error($html_dir,\@error);
		exit;
	}

	print $template->output;
}else{
	my $link = $ipa . '?' . $ENV{'QUERY_STRING'};
	print $q->redirect($link);
}
